const Clutter = imports.gi.Clutter;
const Gio = imports.gi.Gio;
const GObject = imports.gi.GObject;
const Shell = imports.gi.Shell;
const St = imports.gi.St;
const Util = imports.misc.util;

const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const Panel = imports.ui.panel;

// Shell minor version
const Config = imports.misc.config;
const SHELL_MINOR = parseInt(Config.PACKAGE_VERSION.split('.')[1]);

// Cylance
const _txtDomain = 'cylance';
const _localePath = '/opt/cylance/desktop/locales';
const Gettext = imports.gettext;
const GettextDomain = imports.gettext.domain(_txtDomain);
const _ = GettextDomain.gettext;
const N_ = function(x) { return x; }

const _yellow = "yellow";
const _red    = "red";
const _green  = "green";

// Cylance URLs 
const _urlCylance = 'http://www.cylance.com';
const _urlSupport = 'https://support.cylance.com';

// Client UI DBus Interface
const _uiBus = 'com.cylance.protect';
const _uiObj = '/com/cylance/protect/ui';
const _uiIface = '<node> \
<interface name="com.cylance.protect.ui"> \
  <method name="showDetailsWindow"/> \
  <method name="showAboutWindow"/> \
  <method name="getIconColor"> \
      <arg name="value" type="s" direction="out"/> \
  </method> \
  <method name="getShowNotifications"> \
      <arg name="value" type="s" direction="out"/> \
  </method> \
  <method name="setShowNotifications"> \
      <arg name="value" type="s" direction="in"/> \
  </method> \
  <signal name="iconChanged"/> \
  <signal name="showNotificationsChanged"/> \
</interface> \
</node>';

const _uiProxy = Gio.DBusProxy.makeProxyWrapper(_uiIface);

// Cylance commands
const _startUICmd = "/opt/cylance/desktop/cylanceui &> /dev/null &"
const _checkUpdateCmd = "/opt/cylance/desktop/cylance -u &> /dev/null &"
const _checkPolicyUpdateCmd= "/opt/cylance/desktop/cylance -p &> /dev/null &"

let _protectIconGreen;
let _protectIconRed;
let _protectIconYellow;
let _cylanceProtectMenu;

// CylanceAction class
class CylanceAction 
{
    constructor(name, action, param) {
        this._name   = name;
        this._action = action;
        this._param  = param;
    }

    destroy() {}
}

if (SHELL_MINOR >= 34) {
    // CylanceMenuItem class
    var CylanceMenuItem = GObject.registerClass(class CylanceMenuItem extends PopupMenu.PopupBaseMenuItem
    {
        _init(action) {
        super._init();
            this._action = action
            this._label = new St.Label({text: action._name});
            this.add_child(this._label);
        }

        destroy() {
             super.destroy();
        }

        activate(event) {
            this._action._action(event.get_time(), this._action._param);
            super.activate(event);
        }

        _isFunction(obj) {
           return !!(obj && obj.constructor && obj.call && obj.apply);
        }
    });

}else{

    // CylanceMenuItem class
    class CylanceMenuItem extends PopupMenu.PopupBaseMenuItem
    {
        constructor(action) {
        super();
           this._action = action
           this._label = new St.Label({text: action._name});
            this.actor.add_child(this._label);
        }

        destroy() {
           super.destroy();
        }

        activate(event) {
            this._action._action(event.get_time(), this._action._param);
            super.activate(event);
        }

        _isFunction(obj) {
            return !!(obj && obj.constructor && obj.call && obj.apply);
        }
   } // end of class
} // end if


if (SHELL_MINOR >= 34) {
    // CylanceSwitchMenuItem class
    var CylanceSwitchMenuItem = GObject.registerClass(class CylanceSwitchMenuItem extends PopupMenu.PopupSwitchMenuItem
    {

        _init(action) {
        super._init(action._name, action._param);

        this._action = action;

        this.connect('toggled', () => {
            this._action._action(this._switch.state);
            });
        }

        destroy() {
            super.destroy();
        }

        activate(event) {
           super.activate(event);
        }
    });

}else{

// CylanceSwitchMenuItem class
    class CylanceSwitchMenuItem extends PopupMenu.PopupSwitchMenuItem
    {

        constructor(action) {
        super(action._name, action._param);

        this._action = action;

        this.connect('toggled', () => {
            this._action._action(this._switch.state);
            });
        }

        destroy() {
           super.destroy();
        }

        activate(event) {
            super.activate(event);
       }
    } // end of class

} //end if


/////////////////////////////////////////////////////////////////////////////////////////////
// CylanceMenu class

class CylanceMenu extends PanelMenu.Button 
{
    _init ()
    {
        super._init(0.0, "CylanceMenu");
        this.name = "CylanceMenu";

        if (SHELL_MINOR >= 38) {
            this.button = new St.Bin({ style_class: 'panel-button', reactive: true, can_focus: true, x_expand: true, y_expand: false, track_hover: true });
        }else{
            this.button = new St.Bin({ style_class: 'panel-button', reactive: true, can_focus: true, x_fill: true, y_fill: false, track_hover: true });
        }

        this._icon = new St.Icon({ gicon: _protectIconYellow, style_class: 'system-status-icon' });
        this.button.set_child(this._icon);

        // create dbus uiProxy
        this.dbus_proxy = new _uiProxy(Gio.DBus.session, _uiBus, _uiObj, (result, error) => {
            this._dbusClientReady(result, error); 
        });

        let hbox = new St.BoxLayout({style_class: 'panel-status-menu-box'});
        hbox.add_child(this.button);
        if (SHELL_MINOR < 34) {
           this.actor.add_actor(hbox);
        }else{
           this.add_actor(hbox);
        }

        // create menu item "ShowDetails"
        let section = new PopupMenu.PopupMenuSection();
        this._createMenuItem(section, _("ShowDetailsMenu"), () => { this._showDetailsWindow(); });
        this.menu.addMenuItem(section);
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());

        // create other menu items 
        section = new PopupMenu.PopupMenuSection();
        this._createMenuItem(section, _("CM_VisitSite"), this._launchWWW, _urlCylance);
        this._createMenuItem(section, _("CM_Help"), this._launchWWW, _urlSupport);
        this._createMenuItem(section, _("CheckForUpdates"), () => { this._checkForUpdates(); });
        this._createMenuItem(section, _("CM_CheckForPolicyUpdate"), () => { this._checkForPolicyUpdates(); });
        this._createMenuItem(section, _("CM_About"), () => { this._showAboutWindow(); });
        this.menu.addMenuItem(section);
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());

        // create switch menu item "ShowNotifications"
        section = new PopupMenu.PopupMenuSection();
        this._notifyMenuItem = this._createSwitchMenuItem(section, _("CM_ShowNotifications"), enabled => { this._showNotificationToggled(enabled); }, false);
        this.menu.addMenuItem(section);
    }

    destroy() {
        super.destroy();
    }

    _createMenuItem(section, name, cb, param) {
        let action = new CylanceAction(name, cb, param);
        let item   = new CylanceMenuItem(action);
        section.addMenuItem(item);
        return item;
    }

    _createSwitchMenuItem(section, name, cb, param) {
        let action = new CylanceAction(name, cb, param);
        let item   = new CylanceSwitchMenuItem(action);
        section.addMenuItem(item);
        return item;
    }

    _launchWWW(timestamp, url) {
        let launchContext = global.create_app_launch_context(timestamp, -1);

        try {
            Gio.AppInfo.launch_default_for_uri(url, launchContext);
        } catch(e) {
            Main.notifyError(_("Failed to open \"%s\"").format(this.name), e.message);
        }
    }

    _showDetailsWindow() {
        this.dbus_proxy.showDetailsWindowRemote();
    }

    _showAboutWindow() {
        this.dbus_proxy.showAboutWindowRemote();
    }

    _showNotificationToggled(enabled) {
        this.dbus_proxy.setShowNotificationsRemote((enabled == true ? "true" : "false"));
    }

    _showNotificationsChanged(proxy) {
        proxy.getShowNotificationsRemote(enable => { 
            this._notifyMenuItem.setToggleState((enabled == "true" ? true : false));
    });
    }

    _updateShowNotification(enabled) {
        this._notifyMenuItem.setToggleState((enabled == "true" ? true : false));
    }

    _checkForUpdates() {
        Util.spawn(['/bin/bash', '-c', _checkUpdateCmd]);
    }

    _checkForPolicyUpdates() {
        Util.spawn(['/bin/bash', '-c', _checkPolicyUpdateCmd]);
    }

    _iconChanged(proxy) {
        proxy.getIconColorRemote(color => { this._setIcon(color) });
    }

    _setIcon(color) {
        this._icon.gicon = this._getFileIcon(color);
    }


    _getFileIcon(color) {
        if(color == _red)
            return _protectIconRed;
        else if(color == _green)
            return _protectIconGreen;
        else
            return _protectIconYellow;
    }

    _dbusClientReady(result, error) 
    {
        if(error) { log('error: ' + error); }

        this.dbus_proxy.getShowNotificationsRemote(enable => { 
            this._updateShowNotification(enable); 
        });
        
        this.dbus_proxy.getIconColorRemote(color => { 
            this._icon.gicon = this._getFileIcon(color); 
        });

        // connect to ui dbus signals 
        this.dbus_proxy.connectSignal("iconChanged", proxy => { 
            this._iconChanged(proxy); 
        });
        
        this.dbus_proxy.connectSignal("showNotificationsChanged", proxy => { 
            this._showNotificationsChanged(proxy); 
        });
    }
}

if (SHELL_MINOR > 30) {
    CylanceMenu = GObject.registerClass({GTypeName: 'CylanceMenu'},CylanceMenu);
}

/////////////////////////////////////////////////////////////////////////////////////////////
// Cylance Protect extension functions

function init() 
{
    Gettext.bindtextdomain(_txtDomain, _localePath);
}

function enable() 
{
    log('> Cylance extension - enable...');

    Util.spawn(['/bin/bash', '-c', _startUICmd]);

    let iconFile = Gio.File.new_for_path('/opt/cylance/desktop/icon_green.png');
    _protectIconGreen  = new Gio.FileIcon({ file: iconFile });

    iconFile = Gio.File.new_for_path('/opt/cylance/desktop/icon_red.png');
    _protectIconRed    = new Gio.FileIcon({ file: iconFile });

    iconFile = Gio.File.new_for_path('/opt/cylance/desktop/icon_yellow.png');
    _protectIconYellow = new Gio.FileIcon({ file: iconFile });

    _cylanceProtectMenu = new CylanceMenu();
    Main.panel.addToStatusArea('cylance-protect', _cylanceProtectMenu, 0, 'right');

    log('< Cylance extension - enable completed');
}

function disable() 
{
    if (_cylanceProtectMenu !== null) 
    {
        _cylanceProtectMenu.destroy();
        _cylanceProtectMenu = null;
    }
}
